#function list
#newisis
#NeworacleFDR
#pvalue1
#pvalue2
#sis
#gendata
#output

newisis <- function(x,y,initID,sigma2,fdp,types=1,controltype="Fan",tlevel=NULL,iter=10) {
   #if (is.null(initID)) {return(NULL)}
   histID=list(initID)
   
   for (i in 1:iter)
   {  currentID = NeworacleFDR(x,y,initID,sigma2,fdp,types,controltype=controltype,tlevel=tlevel)
      #print(currentID)
      for (k in 1:length(histID)) {
         if (setequal(currentID,histID[[k]]))
            {return(currentID)}
      }
      histID=c(list(currentID),histID)
      initID=currentID
   }
   print("max iteration achieved")
   return(currentID)
}

#use different methods to control FDR
NeworacleFDR <- function(x,y,initID,sigma2,fdp,types=1,controltype="Fan",tlevel=NULL)       #controltype can be "BH1","BH2", "FWER" "Fan"  #tlevel: number of thresholds to check for Fan's procedure
{
 n <- dim(x)[1]
 p <- dim(x)[2]
# procedure 1 OR 2 to caluclate p values
   if (types==1) {newp <- pvalue1(x,y,initID,sigma2)}
   if (types==2) {newp <- pvalue2(x,y,initID,sigma2)}
   newbeta = newp$beta_cond
   newcov = newp$cov
   newpvalues = newp$pvalues
   sortedpvalues = sort(newpvalues,decreasing=F)
   totalselectID = NULL
   for (k in 1:length(controltype)) 
   { 
      currentcontroltype=controltype[k]   
      thresh = 0
      if (currentcontroltype=="BH1") {
           if (sum(sortedpvalues*p/(1:p)<=fdp)>0)                                             #BenjaminiHochberg procedure
           {thresh = max(sortedpvalues[sortedpvalues*p/(1:p)<=fdp])}
      }
      if (currentcontroltype=="BH2") {                                              #BenjaminiHochbergYekutieli procedure
         if (p<1000) {tempc=sum(1/(1:p))}
            else {tempc=log(p)+(sum(1/(1:1000))-log(1000))}
         if (sum(sortedpvalues*p*tempc/(1:p)<=fdp)>0)   
            {thresh = max(sortedpvalues[sortedpvalues*p*tempc/(1:p)<=fdp])}
      }
      if (currentcontroltype=="FWER") { thresh=fdp/p }
      if (currentcontroltype=="Fan") {
          if (is.null(tlevel)) {tt = sortedpvalues[1:min(max(20,n/4),p)]}
             else  {tt = sortedpvalues[1:min(tlevel,p)]}
          out = pfa.test(X=newbeta,Sigma=newcov,tval=tt)
          cutoff = sum(out$FDP$FDP<=fdp)
          if (cutoff >0)
          thresh = out$FDP$t[cutoff]
      }
       
       selectid= NULL
       if (sortedpvalues[1]<=thresh) { selectid = (1:p)[newpvalues<=thresh] }
       totalselectID=c(totalselectID,list(selectid))
    }
    if (length(totalselectID)==1) {return(totalselectID[[1]])}
       else {return(totalselectID)}
       
}


pvalue1 <- function(x,y,index,sigma2)      # replace oraclep in the near future, note that there is no pvalues2 anymore.
{#index is based on some pre-selection procedure
 y = y-mean(y)
 x = scale(x, center = TRUE, scale = TRUE)
 n <- dim(x)[1]
 p <- dim(x)[2]
 pvalues <- rep(0,p)
 beta_cond <- rep(0,p)
 cov=matrix(0,p,p)
 xres <- x

 # compute beta_cond and covariance matrix of beta_cond
 if (!is.null(index)) {
    #Hmat = x[,index]%*%ginv(t(x[,index])%*%x[,index])%*%t(x[,index])
    Hmat = x[,index]%*%solve(t(x[,index])%*%x[,index])%*%t(x[,index])
    xres[,-index] = (diag(1,n)-Hmat)%*%x[,-index]

    for (j in 1:length(index))
    { k = index[j]
      if (length(index)>1)
      {xnoj = x[,index[-j]]
       xres[,k] = x[,k]-xnoj%*%solve(t(xnoj)%*%xnoj)%*%t(xnoj)%*%x[,k]
       #xres[,k] = x[,k]-xnoj%*%ginv(t(xnoj)%*%xnoj)%*%t(xnoj)%*%x[,k]
      }
      
    }
 }
 beta_cond = t(xres)%*%y/colSums(xres*x)
 xcov = t(xres)%*%xres
 xvar = diag(xcov)%*%t(diag(xcov))
 cov =  sigma2*xcov/xvar
 # compute pvalues2 based on Fan
 pvalues = 2*(1-pnorm(abs(beta_cond)/sqrt(diag(cov))))
 return(list(beta_cond=beta_cond,cov=cov,pvalues=pvalues))
}


pvalue2 <- function(x,y,index,sigma2,cop=NULL)           #implement procedure 2 and 3.
{#index is based on some pre-selection procedure
 y = y-mean(y)
 x = scale(x, center = TRUE, scale = TRUE)

 p <- dim(x)[2]
 pvalues <- rep(0,p)
 beta_cond <- rep(0,p)
 cov=matrix(0,p,p)
 xres <- x
 if (is.null(cop)) {cop=max(1,floor(log(p)))}
 Cor = cor(x)

 # compute beta_cond and covariance matrix of beta_cond
 for (j in 1:p)
    {
    index2= order(abs(Cor[j,]),decreasing=TRUE)[1:(cop+1)]  # j itself and cop most correlated variables
    tindex=setdiff(union(index,index2),j)
    xnoj = x[,tindex]
    #xres[,j] = x[,j]-xnoj%*%solve(t(xnoj)%*%xnoj)%*%t(xnoj)%*%x[,j]
    xres[,j] = x[,j]-xnoj%*%ginv(t(xnoj)%*%xnoj)%*%t(xnoj)%*%x[,j]
    }


 beta_cond = t(xres)%*%y/colSums(xres*x)
 xcov = t(xres)%*%xres
 xvar = diag(xcov)%*%t(diag(xcov))
 cov =  sigma2*xcov/xvar
 # compute pvalues2 based on Fan
 pvalues = 2*(1-pnorm(abs(beta_cond)/sqrt(diag(cov))))
 return(list(beta_cond=beta_cond,cov=cov,pvalues=pvalues))
}


oraclep <- function(x,y,index,sigma2)
{#index is based on some pre-selection procedure  
 y = y-mean(y)
 x = scale(x, center = TRUE, scale = TRUE)
 p <- dim(x)[2]
 pvalues <- rep(0,p)
 beta_cond <- rep(0,p)
 cov=matrix(0,p,p)
 xres <- x
 #for selected variables
 if (!is.null(index)) {                                      # different treatment for NULL index
     beta_cond[index] <- summary(lm(y~x[,index]-1))$coef[,1]
     pvalues[index] <- summary(lm(y~x[,index]-1))$coef[,4]  #noise variance estimated based on the reduced model
 }
 #for noise variables
 for (j in 1:p)
 {if (sum(j==index)==0) 
   {beta_cond[j] = summary(lm(y~x[,c(j,index)]-1))$coef[1,1]
    pvalues[j]= summary(lm(y~x[,c(j,index)]-1))$coef[1,4]
   }
 }
return(pvalues)
}



sis <- function(x,y,k0=NULL)
{ n <- dim(x)[1]
  if (is.null(k0)) {k0 <- ceiling(n/log(n))}
  mycor <- abs(cor(x,y))
  id <- order(abs(cor(x,y)),decreasing=T)[1:k0]
  return(id)
}                                                                   

gendata <- function(n,p,imp,betacoef=NULL,case,rho=0.5,R2)       #case=1 IID; 2 AR; 3 CS; more?
{
   p0 = length(imp);                
   z <- matrix(rnorm(n*p),n,p)

#generate x 
   x=z

#AR(0.5)  
   if (case==2)  
   {#rho=0.5  
     for (j in 2:p)
     {x[,j]=rho*x[,j-1]+sqrt(1-rho^2)*x[,j]}
   }
   
#CS
   if (case==3)  
   { w=rnorm(n,0,1)                                                                                                
     for (j in 1:p)
		  {x[,j]=(x[,j]+sqrt(rho/(1-rho))*w)*sqrt(1-rho)}
    }
    
#generate y and errors
   beta0 = rep(0,p)
   if (is.null(betacoef)) {beta0[imp] = rep(1,p0)}
     else  {beta0[imp] = betacoef}
   f = x%*%beta0
   sigma2 = mean(f^2)*(1-R2)/R2
   errs <- rnorm(n,0,sqrt(sigma2))
   y = f+errs
   return(list(x=x,y=y,sigma2=sigma2))
}

stan <- function(x)
{ p <- dim(x)[2]
  stanx <- x
  for (j in 1:p)
    stanx[,j] = (x[,j]-mean(x[,j]))/sqrt(var(x[,j]))
  return(stanx)
}

output <- function(selectID, imp)
{ truepos = length(intersect(selectID,imp))
  falsepos =  length(selectID)-truepos
  fdr = 0
  if (length(selectID)> 0)
  {fdr= 1-truepos/length(selectID)}
  return(c(truepos, falsepos, fdr))
}

#compute fdr
fdr <- function(selectID, imp)
{ truepos = length(intersect(selectID,imp))
  fdr = 0
  if (length(selectID)> 0)
  {fdr= 1-truepos/length(selectID)}
  return(fdr)
}

#bic
 bic <- function(x,y,tmpModel)
 {  n = dim(x)[1]
    df = length(tmpModel)  
    mx = x[,tmpModel]
    ols = lm(y~mx-1)$coef
    if (df>1)
     {rss = mean((y-mx%*%ols)^2)}
    else
     {rss = mean((y-mx*ols)^2)}
    BIC = log(rss)+df*log(n)/n
    return(BIC)
 } 

#my rcv
myrcv <- function(x, y) {
  n <- nrow(x)
  p <- ncol(x)
  # dimension reduction by removing ultra-highly correlated variables
  # C <- abs(cor(x,x))

  sigma_sq <- sigma1_sq <- sigma2_sq <- rep(0,30)
  for (i in 1:30){

      seq <- c(1:n)
      seq <- sample(seq)
      n2 = floor(n/2)
      seq1 <- seq[1:n2]
      seq2 <- seq[(n2+1):n]	# random split of the data into two halves
      y1 <- y[seq1]
      x1 <- x[seq1,]
      y2 <- y[seq2]
      x2 <- x[seq2,]

     ind1 <- sis(x1,y1,k0=max(n/log(n),floor(n/4))) 
     resid1 <- lsfit(x2[,ind1],y2)$resid
     sigma1_sq[i] <- sum(resid1^2)/(n-n2 - length(ind1))
     #print(sort(ind1));print(sigma1_sq[i])
     
     ind2 <- sis(x2,y2,k0=max(n/log(n),floor(n/4)))  
     resid2 <- lsfit(x1[,ind2],y1)$resid
     sigma2_sq[i] <- sum(resid2^2)/(n2 - length(ind2))
     #print(sort(ind2));print(sigma2_sq[i])
      
    sigma_sq[i] <- (sigma1_sq[i] + sigma2_sq[i])/2	   # RCV-estimated variance
  }
  sigmahat=sqrt(mean(sigma_sq[sigma_sq!=0]))
  return(sigmahat)
}

#glmnet
myglmnet <- function(x,y)
{
fit1 <- glmnet(x,y)
cv1 <- cv.glmnet(x,y)
#s1 <- cv1$lambda.1se
s1 <- cv1$lambda.min
coef1 <- coef(fit1,s=s1)[-1]
var1 <- which(abs(coef1)>0)
list(var=var1,coef=coef1)
}

