source("core20131119.r")
#source("pfa1.01.r")
library(hdlm)
library(hdi)
library(svd)
library(MASS)
library(gap)
library(glmnet)
      
#########
#simulation setting: cov=AR(0.1,0.5,0.9) n,p=100,500 and 200,1000 important variables 1,2,3,5,8,13,21,34,55,89 beta0=1,-1,...
# R2=0.5,0.7,0.9
# one step procedure: initial= 1) true important variables (oracle) 2) naive SIS (first n/log n) 3)NULL 4)FanHanGu
                      # FDR control: BH2 and Fan
# iterative procedure: initial= 1)naive SIS 2) NULL FDR control = BH2
# (one step) type 2 procedure initial = 1) naive SIS 2) NULL # FDR control: BH2 and Fan
# Split data: SIS on first half and one step procedure for the second   # FDR control: BH2 and Fan

OPvalue <- function(n,p,imp,betacoef=NULL,rho,R2,case,Num,seed0)    
{
opv = matrix(0,Num,p)
#pv1 and pv2 are procedure 1 and 2 based on SIS with 
#k0=max(20,n/log(n))
#pv3 is procedrue 1 based on glmnet with cv tuning
pv1 = opv
pv2 = opv
pv3 = opv
opvRK = opv
pv1RK = opv
pv2RK = opv
pv3RK = opv
# added 2017-03
pv4 =pv4RK = opv
pv5 =pv5RK = opv
for (j in 1:Num) 
{                                       #controltype can be "BH1","BH2", "FWER" "Fan"  #tlevel: number of
print(j)
set.seed(seed0+j)
mydata <- gendata(n,p,imp,betacoef=betacoef,case=2,rho=rho,R2=R2)
x <- mydata$x
y <- mydata$y
#truesigma2 <- mydata$sigma2
#rcvsigma2 = rcv(x,y)^2 
#Fan's estimated sigma by rcv 
#always over-estimated here (in the function pfa1.01.r)
myrcvsigma2 = myrcv(x,y)^2 
#estimated sigma by rcv always over-estimated here
oraclesigma2 = sum(lsfit(x[,imp],y)$resid^2)/(n-length(imp)) 
# estimated sigma based on the oracle model using OLS

# conduct SIS
#sisID <- sis(x,y)
#split the data into halved, conduct SIS/lasso with the data in #id1. Use x2 and y2 to fit the data
id1 <- sample(1:n,size=floor(n/2),replace=F)
x2 <- x[-id1,]
y2 <- y[-id1]

sisID2 <- sis(x[id1,],y[id1],k0=min(20,ceiling(n/log(n))))
lassoID <- myglmnet(x[id1,],y[id1])$var
if (sum(lassoID)==0) lassoID=NULL

#using myrcvsigma2
opv[j,] <- pvalue1(x2,y2,index=imp,sigma2=oraclesigma2)$pvalues
pv1[j,] <- pvalue1(x2,y2,index=sisID2,sigma2=myrcvsigma2)$pvalues
pv2[j,] <- pvalue2(x2,y2,index=sisID2,sigma2=myrcvsigma2,cop=10)$pvalues
pv3[j,] <- pvalue1(x2,y2,index=lassoID,sigma2=myrcvsigma2)$pvalues
pv4[j,] <- hdi(x,y,method = "multi.split")$pval.corr
pv5[j,] <- ridge.proj(x, y)$pval.corr


# get ranks
opvRK[j,] = rank(opv[j,])
pv1RK[j,] = rank(pv1[j,])
pv2RK[j,] = rank(pv2[j,])
pv3RK[j,] = rank(pv3[j,])
pv4RK[j,] = rank(pv4[j,])
pv5RK[j,] = rank(pv5[j,])
}
return(list(opv=opv,pv1=pv1,pv2=pv2,pv3=pv3,pv4=pv4,pv5=pv5,opvRK=opvRK,pv1RK=pv1RK, pv2RK=pv2RK, pv3RK=pv3RK, pv4RK=pv4RK, pv5RK=pv5RK))
}

mysummary <- function(Ops,p)
{
test_stat=matrix(0,6,p)
test_pvalue=matrix(0,6,p)
ranks = matrix(0,6,p)
for (j in 1:p)
{
test_stat[1,j] <- ks.test(Ops$opv[,j],"punif")$statistic
test_stat[2,j] <- ks.test(Ops$pv1[,j],"punif")$statistic
test_stat[3,j] <- ks.test(Ops$pv2[,j],"punif")$statistic
test_stat[4,j] <- ks.test(Ops$pv3[,j],"punif")$statistic
test_stat[5,j] <- ks.test(Ops$pv4[,j],"punif")$statistic
test_stat[6,j] <- ks.test(Ops$pv5[,j],"punif")$statistic
test_pvalue[1,j] <- ks.test(Ops$opv[,j],"punif")$p.value
test_pvalue[2,j] <- ks.test(Ops$pv1[,j],"punif")$p.value
test_pvalue[3,j] <- ks.test(Ops$pv2[,j],"punif")$p.value
test_pvalue[4,j] <- ks.test(Ops$pv3[,j],"punif")$p.value
test_pvalue[5,j] <- ks.test(Ops$pv4[,j],"punif")$p.value
test_pvalue[6,j] <- ks.test(Ops$pv5[,j],"punif")$p.value
}
ranks[1,] = apply(Ops$opvRK,2,median)
ranks[2,] = apply(Ops$pv1RK,2,median)
ranks[3,] = apply(Ops$pv2RK,2,median)
ranks[4,] = apply(Ops$pv3RK,2,median)
ranks[5,] = apply(Ops$pv4RK,2,median)
ranks[6,] = apply(Ops$pv5RK,2,median)


return(list(test_stat=test_stat,test_pvalue=test_pvalue, ranks=ranks))
}

###
############################################
n <- 200;p <- 1000;imp =c(1,2,3,5,8);betacoef=c(1,1,2,2,-1);R2=0.6;Num <- 3
 

### Example 1 (independent)
case<-2;rho=0;seed0=200;
Ops1 = OPvalue(n, p, imp, betacoef, rho,R2, case, Num,seed0)
result1 = mysummary(Ops1,p)
### Example 2 (AR)
rho=0.5;seed0=500
Ops2 = OPvalue(n, p, imp, betacoef,rho,R2, case, Num,seed0)
result2 = mysummary(Ops2,p)

#save(Ops1, Ops2, file = "example1.RData")
save(Ops1, Ops2, file = "example120170410.RData")

  
 
